/*
 * Project name:
     MPU_IMU click (Using mikroE's MPU_IMU click board)
 * Copyright:

 * Revision History:

 * Description:

 * Test configuration:
     MCU:             PIC32 - P32MX795F512L

     Dev.Board:       EasyPIC7

     Oscillator:      80.0000 MHz
     ext. modules:    mpu_imu click board - 
     SW:              mikroC PRO for PIC32

 * NOTES:
     - Place MPU_IMU click board in the mikroBUS socket 1 on the EasyPIC v7 board.
     - RA2 & RA3 PULL UP position !!!
 */

#include "MPU_IMU_Register_Map.h"
#include "MPU_IMU_I2C.h"

// Gyro offset
#define Gyrox_offset  3.3
#define Gyroy_offset  0
#define Gyroz_offset  0

// Accel offset
#define Accelx_offset -0.07
#define Accely_offset 0.02
#define Accelz_offset 0.2

// Temperature offset
#define Temp_offset   36.53

// MPU IMU module connections
sbit INT at RE8_bit;
sbit INT_Direction at TRISE0_bit;
// End of MPU IMU module connections

// Variables used
unsigned int raw_data[7];
int gyro_x_temp, gyro_y_temp, gyro_z_temp, accel_x_temp, accel_y_temp, accel_z_temp, temp_raw;
char i, buff, gyro_x_out[15], gyro_y_out[15], gyro_z_out[15], accel_x_out[15], accel_y_out[15], accel_z_out[15], temp_out[15];
float temp, gyro_x, gyro_y, gyro_z, accel_x, accel_y, accel_z;

// Initialize MPU
void Init_MPU(){
  unsigned char dat;

  dat = 0x6c;
  MPU_I2C_Write(mpu_I2C_ADDR, mpu_rm_PWR_MGMT_1 , 1, &dat);
  Delay_ms(100);
  
  dat = 0x00;
  MPU_I2C_Write(mpu_I2C_ADDR, mpu_rm_PWR_MGMT_1 , 1, &dat);      //   MPU_I2C_Write(mpu_I2C_ADDR, mpu_rm_PWR_MGMT_1 , 1, 0x00);
  Delay_ms(100);

  dat = 0x69;
  MPU_I2C_Write(mpu_I2C_ADDR, mpu_rm_FIFO_EN , 1, &dat);         //   MPU_I2C_Write(mpu_I2C_ADDR, mpu_rm_FIFO_EN , 1, 0x78);
  Delay_ms(100);
  
  dat = 0xA1;
  MPU_I2C_Write(mpu_I2C_ADDR, mpu_rm_INT_ENABLE , 1, &dat);      //     MPU_I2C_Write(mpu_I2C_ADDR, mpu_rm_INT_ENABLE , 1, 0x10);
  Delay_ms(100);
  MPU_I2C_Read (mpu_I2C_ADDR, mpu_rm_ACCEL_CONFIG, 1, &raw_data);
  Delay_ms(100);
  raw_data[0] |= 0x18;
  MPU_I2C_Write(mpu_I2C_ADDR, mpu_rm_ACCEL_CONFIG, 1, &raw_data[0]);
  Delay_ms(100);
  MPU_I2C_Read (mpu_I2C_ADDR, mpu_rm_INT_ENABLE, 1, &raw_data);
  Delay_ms(100);
  raw_data[0] |= 0x11;
  MPU_I2C_Write(mpu_I2C_ADDR, mpu_rm_INT_ENABLE, 1, &raw_data[0]);
  Delay_ms(100);
}



// Extract readings from MPU
void Extract_Readings() {
  // Extract accel readings
  accel_x_temp = raw_data[0];
  accel_x = (float)accel_x_temp / 2048.0;
  accel_y_temp = raw_data[1];
  accel_y = (float)accel_y_temp / 2048.0;
  accel_z_temp = raw_data[2];
  accel_z = (float)accel_z_temp / 2048.0;

  // Extract temperature readings
  temp_raw = raw_data[3];
  temp = (float)temp_raw / 340.0;

  // Extract gyro readings
  gyro_x_temp = raw_data[4];
  gyro_x = (float)gyro_x_temp / 131.0;
  gyro_y_temp = raw_data[5];
  gyro_y = (float)gyro_y_temp / 131.0;
  gyro_z_temp = raw_data[6];
  gyro_z = (float)gyro_z_temp / 131.0;
}

// Set offset for the readings
void Set_Offset() {
  // Set accel offset
  accel_x += Accelx_offset;
  accel_y += Accely_offset;
  accel_z += Accelz_offset;

  // Set temp offset
  temp += Temp_offset;

  // Set gyro offset
  gyro_x += Gyrox_offset;
  gyro_y += Gyroy_offset;
  gyro_z += Gyroz_offset;
}

// Convert readings to string
void Convert_Readings_To_String() {
  // Convert accel readings
  FloatToStr(accel_x, accel_x_out);
  FloatToStr(accel_y, accel_y_out);
  FloatToStr(accel_z, accel_z_out);

  // Convert temperature readings
  FloatToStr(temp, temp_out);

  // Convert gyro readings
  FloatToStr(gyro_x, gyro_x_out);
  FloatToStr(gyro_y, gyro_y_out);
  FloatToStr(gyro_z, gyro_z_out);
}

// Print readings on UART
void Print_Readings() {
  // Print gyro readings
  UART2_Write_Text("Gyro readings :");
  UART2_Write(0x0D);
  UART2_Write_Text("x = ");
  UART2_Write_Text(gyro_x_out);
  UART2_Write(0x0D);
  UART2_Write_Text("y = ");
  UART2_Write_Text(gyro_y_out);
  UART2_Write(0x0D);
  UART2_Write_Text("z = ");
  UART2_Write_Text(gyro_z_out);
  UART2_Write(0x0D);
  UART2_Write(0x0D);
  UART2_Write(0x0D);

  // Print accel readings
  UART2_Write_Text("Accel readings :");
  UART2_Write(0x0D);
  UART2_Write_Text("x = ");
  UART2_Write_Text(accel_x_out);
  UART2_Write(0x0D);
  UART2_Write_Text("y = ");
  UART2_Write_Text(accel_y_out);
  UART2_Write(0x0D);
  UART2_Write_Text("z = ");
  UART2_Write_Text(accel_z_out);
  UART2_Write(0x0D);
  UART2_Write(0x0D);
  UART2_Write(0x0D);

  // Convert temperature readings
  UART2_Write_Text("Temperature readings (Celsius) = ");
  UART2_Write_Text(temp_out);
  UART2_Write(0x0D);
  UART2_Write(0x0D);
  UART2_Write(0x0D);
  Delay_ms(1000);
}

void main() {
  AD1PCFG = 0xFFFF;                          // Configure AN pins as digital I/O

  // Set PORTC as output
  TRISC = 0;
  TRISE = 0;

  // Set IN pin as input
  INT_Direction = 1;

  // Initialize I2C module
  I2C2_Init(400000);
  Delay_ms(100);

  // Initialize UART module
  UART2_Init(115200);
  Delay_ms(100);
  UART2_write_text("uController started\x0D");
 
 
  Init_MPU();                    // Initialize MPU

  while(1) {
    while(INT != 1)              // Wait until INT line becomes 1
      ;
    MPU_I2C_Read_Int(mpu_I2C_ADDR, mpu_rm_ACCEL_XOUT_H, 7, &raw_data);   // Read data from MPU
    Extract_Readings();                                                  // Extract data from the readings
    Set_Offset();                                                        // Set offset
    Convert_Readings_To_String();                                        // Convert readings to strings
    Print_Readings();                                                    // Print readings on UART
  }
}